/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2014 - 2017 Dries K. Aka Dries007
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package net.dries007.holoInventory.client.renderers;

import net.dries007.holoInventory.client.ClientEventHandler;
import net.minecraft.client.Minecraft;
import net.minecraft.client.multiplayer.WorldClient;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.RenderItem;
import net.minecraft.client.renderer.entity.RenderManager;
import net.minecraft.client.renderer.tileentity.TileEntityRendererDispatcher;
import net.minecraft.client.resources.I18n;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.math.Vec3d;
import net.minecraft.village.MerchantRecipe;
import net.minecraft.village.MerchantRecipeList;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.List;

@SideOnly(Side.CLIENT)
public class MerchantRenderer implements IRenderer
{
    private final String name;
    private final List<MerchantRecipe> recipes;

    public MerchantRenderer(String name, MerchantRecipeList input)
    {
        this.name = I18n.func_135052_a(name);
        this.recipes = input;
    }

    @Override
    public void render(WorldClient world, RayTraceResult hit, Vec3d pos)
    {
        Minecraft mc = Minecraft.func_71410_x();
        RenderManager rm = mc.func_175598_ae();
        RenderItem ri = mc.func_175599_af();

        GlStateManager.func_179137_b(pos.field_72450_a - TileEntityRendererDispatcher.field_147554_b, pos.field_72448_b - TileEntityRendererDispatcher.field_147555_c, pos.field_72449_c - TileEntityRendererDispatcher.field_147552_d);

        GlStateManager.func_179114_b(-rm.field_78735_i, 0.0F, 0.5F, 0.0F);
        GlStateManager.func_179114_b(rm.field_78732_j, 0.5F, 0.0F, 0.0F);
        GlStateManager.func_179137_b(0, 0, -0.5);

        double d = pos.func_72438_d(new Vec3d(TileEntityRendererDispatcher.field_147554_b, TileEntityRendererDispatcher.field_147555_c, TileEntityRendererDispatcher.field_147552_d));

        if (d < 1.75) return;
        GlStateManager.func_179139_a(d * 0.2, d * 0.2, d * 0.2);

        // Draw name, with depth disabled
        {
            GlStateManager.func_179094_E();
            GlStateManager.func_179123_a();
            GlStateManager.func_179114_b(180, 0, 0, 1);
            GlStateManager.func_179137_b(0, -0.6f -0.4f * (recipes.size()/2.0), 0);
            //GlStateManager.translate(0, -1 - 0.5f * (rows/2.0), 0);
            GlStateManager.func_179139_a(0.03, 0.03, 0.03);
            int w = mc.field_71466_p.func_78256_a(name);
            GlStateManager.func_179097_i();
            mc.field_71466_p.func_78276_b(name,  -w/2, 0, 0xFFFFFF);
            GlStateManager.func_179126_j();
            GlStateManager.func_179099_b();
            GlStateManager.func_179121_F();
        }

        for (int row = 0; row < recipes.size(); row ++)
        {
            final MerchantRecipe recipe = recipes.get(row);

            GlStateManager.func_179094_E();
            GlStateManager.func_179123_a();

            RenderHelper.renderStack(ri, recipe.func_77394_a(), 4, 0, recipes.size(), row);
            if (recipe.func_77398_c())
                RenderHelper.renderStack(ri, recipe.func_77396_b(), 4, 1, recipes.size(), row);
            RenderHelper.renderStack(ri, recipe.func_77397_d(), 4, 3, recipes.size(), row);

            GlStateManager.func_179099_b();
            GlStateManager.func_179121_F();
        }
        // Draw stack sizes later, to draw over the items (disableDepth)
        GlStateManager.func_179097_i();
        for (int row = 0; row < recipes.size(); row ++)
        {
            final MerchantRecipe recipe = recipes.get(row);

            GlStateManager.func_179094_E();
            GlStateManager.func_179123_a();

            int color = recipe.func_82784_g() ? ClientEventHandler.TEXT_COLOR_LIGHT : ClientEventHandler.TEXT_COLOR;

            RenderHelper.renderName(mc.field_71466_p, recipe.func_77394_a(), 4, 0, recipes.size(), row, color);
            if (recipe.func_77398_c())
                RenderHelper.renderName(mc.field_71466_p, recipe.func_77396_b(), 4, 1, recipes.size(), row, color);
            RenderHelper.renderName(mc.field_71466_p, recipe.func_77397_d(), 4, 3, recipes.size(), row, color);

            GlStateManager.func_179099_b();
            GlStateManager.func_179121_F();
        }
        GlStateManager.func_179126_j();
    }

    @Override
    public boolean shouldRender()
    {
        return recipes.size() != 0;
    }

}
